//
//  URLAppAppDelegate.m
//  URLApp
//
//  Created by Calvin Stephens on 2/23/11.
//  Copyright 2011 Unicode Concepts. All rights reserved.
//

#import "URLAppAppDelegate.h"

@implementation URLAppAppDelegate

@synthesize window;
@synthesize topPaidAppsTableView;
@synthesize topPaidAppsSummaryTextField;

- (id)init
{
	topPaidAppsArray = [[NSMutableArray alloc] init];
	topPaidAppsSummaryArray = [[NSMutableArray alloc] init];
	
	return self;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification 
{
		// Create the request
		// NSURL *url = [NSURL URLWithString:@"http://www.apple.com/"];
	
	
	theRequest = [NSURLRequest requestWithURL:[NSURL URLWithString:@"http://ax.itunes.apple.com/WebObjects/MZStoreServices.woa/ws/RSS/toppaidapplications/limit=10/xml"]
													cachePolicy:NSURLRequestUseProtocolCachePolicy
												timeoutInterval:60.0];
			   
		// Create the connection with the request
		// and start loading the data
	theConnection = [[NSURLConnection alloc] initWithRequest:theRequest
													delegate:self];
	
	if (theConnection) {
			// Create the NSMutableData to hold the recieved data
			// recievedData is an instance variable declared elsewhere
		receivedData = [[NSMutableData data] retain];
		
	} else {
		NSLog(@"thConnection failed");
	}
	
	theNotification = [NSNotification 
					   notificationWithName:@"NSTableViewSelectionDidChangeNotification" 
					   object:topPaidAppsTableView];
	
	[window setBackgroundColor:[NSColor darkGrayColor]];
}


- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
		// This method is called when the server has determined that it
		// has enough information to create the NSURLResponse.
	
		// It can be called multiple times, for example in the case of a
		// redirect, so each time we reset the data
	
		// receivedData is an instance variable declared elsewhere
	
	[receivedData setLength:0];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
		// Append the new data to receivedData
	[receivedData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
		// release the connection, and the data object
	[connection release];
	
		// receivedData is declared as a method instance elsewhere
	[receivedData release];
	
		// inform the user
	NSLog(@"Connection failed! Error - %@ %@",
		  [error localizedDescription], [[error userInfo] objectForKey:NSURLErrorFailingURLStringErrorKey]);
}
								
		  
- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
		// Do something with the data
		// receivedData declared elsewhere
	NSLog(@"Succeded! Recieved %d bytes of data",[receivedData length]);
	
	NSString *xmlCheck = [[[NSString alloc] initWithData:receivedData
												encoding:NSUTF8StringEncoding] autorelease];
	
	NSLog(@"xmlCheck = %@", xmlCheck);
	
	NSXMLParser *parser = [[NSXMLParser alloc] initWithData:receivedData];
	
	[parser setDelegate:self];
	
	[parser parse];
	
	[parser release];
	
	[connection release];
	[receivedData release];
}

#pragma mark -
#pragma mark NSXMLParser Delegate Methods

- (void)parser:(NSXMLParser *)parser 
didStartElement:(NSString *)elementName 
  namespaceURI:(NSString *)namespaceURI 
 qualifiedName:(NSString *)qName 
	attributes:(NSDictionary *)attributeDict
{
	
	if ([elementName isEqual:@"entry"]) {
		NSLog(@"Found a song entry");
		waitingForEntryTitle = YES;
	}
	
	if ([elementName isEqual:@"title"] && waitingForEntryTitle) {
		NSLog(@"Found title");
		
		titleString = [[NSMutableString alloc] init];
	}
	
	if ([elementName isEqual:@"summary"]) {
		summaryString = [[NSMutableString alloc] init];
	}
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string
{
	[titleString appendString:string];
	[summaryString appendString:string];
}

- (void)parser:(NSXMLParser *)parser 
 didEndElement:(NSString *)elementName 
  namespaceURI:(NSString *)namespaceURI 
 qualifiedName:(NSString *)qName
{
	if ([elementName isEqual:@"title"] && waitingForEntryTitle) {
		NSLog(@"Ended Title %@", titleString);
		
		[topPaidAppsArray addObject:titleString];
		
		NSLog(@"topPaidAppsArray: %@", [topPaidAppsArray description]);
		
		[titleString release];
		titleString = nil;
		
		[topPaidAppsTableView reloadData];
	}
	
	if ([elementName isEqual:@"entry"]) {
		NSLog(@"Ended a song entry");
		waitingForEntryTitle = NO;
	}
	
	if ([elementName isEqual:@"summary"]) {
		NSLog(@"Ended Summary: %@", summaryString);
		
		[topPaidAppsSummaryArray addObject:summaryString];
		
		NSLog(@"topPaidAppsSummaryArray: %@", topPaidAppsSummaryArray);
	}
}

#pragma mark -
#pragma mark TODO: Fix Data Display in Table View

- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [topPaidAppsArray count];
}

- (id)tableView:(NSTableView *)aTableView
objectValueForTableColumn:(NSTableColumn *)aTableColumn
			row:(NSInteger)rowIndex
{
	
	return [topPaidAppsArray objectAtIndex:rowIndex];
	
}

- (void)tableView:(NSTableView *)aTableView 
   setObjectValue:(id)anObject 
   forTableColumn:(NSTableColumn *)aTableColumn 
			  row:(NSInteger)rowIndex
{
		// return [topPaidAppsArray  ];
}

- (void)tableViewSelectionDidChange:(NSNotification *)notification
{
	topPaidAppsTableView = [notification object];
	
	NSInteger chosenRow = [[topPaidAppsTableView selectedRowIndexes] firstIndex];
	
	NSMutableString *item = [topPaidAppsSummaryArray objectAtIndex:chosenRow];
	
	[topPaidAppsSummaryTextField setStringValue:item];
}

@end
